/**
   Fetch and validate a location

    Copyright: John Buckley (cocoa.coder@gmail.com)
    July 2005

    DISCLAIMER: This computer program is supplied "AS IS".
    The Author disclaims all warranties, expressed or implied, including,
    without limitation, the warranties of merchantability and of fitness
    for  any purpose.  The Author assumes no liability for direct, indirect,
    incidental, special, exemplary, or consequential damages, which may
    result from the use of the computer program, even if advised of the
    possibility of such damage.  There is no warranty against interference
    with your enjoyment of the computer program or against infringement.
    There is no warranty that my efforts or the computer program will
    fulfill any of your particular purposes or needs.  This computer
    program is provided with all faults, and the entire risk of satisfactory
    quality, performance, accuracy, and effort is with the user.

    LICENSE: Permission is hereby irrevocably granted to everyone to use,
    copy, modify, and distribute this computer program, or portions hereof,
    purpose, without payment of any fee, subject to the following
    restrictions:

    1. The origin of this binary or source code must not be misrepresented.

    2. Altered versions must be plainly marked as such and must not be
    misrepresented as being the original binary or source.

    3. The Copyright notice, disclaimer, and license may not be removed
    or altered from any source, binary, or altered source distribution.
*/

var isValidatingLocation = false;
var validateAnimTimerId = 0;

var menuPopupX = 40;
var menuPopupY = 83;

// 'referrer' is necessary, else we don't get a results page
var searchUrl = "http://news.bbc.co.uk/weather/mobile/search?search=%s&region=world&referrer=%2Fweather%2Fmobile%2Fsearchresults";
var searchResultsRequest = null; // used to fetch extra result pages

function searchFrame() {
    return document.getElementById('search_frame');
}

function validateLocationInput()
{
    var location = $('#location-input').val().strip();
    debug('validateLocationInput: %s', location);

    if ( location == prefs.locationShortName && prefs.locationId )
        return;

    if ( searchResultsRequest )
        searchResultsRequest.abort();

    isValidatingLocation = true;
    searchFrame().onload = searchFrameDidLoad;
    searchFrame().src = searchUrl.format(escape(location));

    // Animate...(the location-caption :)
    if ( !validateAnimTimerId ) {
        validateAnimTimerId = setInterval( function() { loadingAnimation(validateAnimTimerId) }, 500 );
        loadingAnimData[validateAnimTimerId] = { text:"Validating", node:"#location-caption", count:3 };
    }
    loadingAnimation(validateAnimTimerId);
}

function isPostcode(text) {
    return (text && text.match(/\b[a-z]{1,2}[0-9]{1,2}\b/i));
}

function searchFrameDidLoad()
{
    var frame = searchFrame();

    if ( frame.contentDocument.URL == 'about:blank' )
        return;

    if ( frame.contentDocument.readyState != 'complete' ) {
        locationProcessed({errorString:"Request failed"});
        return;
    }

    var results = [];
    try {
        // If there is a single match the forecast URL is returned in the form
        // news.bbc.co.uk/weather/mobile/forecast/<id>
        var id = frame.contentDocument.URL.firstMatch(/\/mobile\/forecast\/([0-9]+)\??/);
        if ( id ) {
            // Extract the location name
            var name = $('.content h1', frame.contentDocument).text();
            if ( name ) {
                var shortName;
                var longName;
                // name may be returned as 'Foo (Forecast for Bar)'
                var m = name.match(/([\w\s,]+) \(Forecast for ([\w\s,]+)/i);
                if ( m && m.length == 3 ) {
                    if ( isPostcode(shortName) ) {
                        shortName = m[2];
                        longName = m[2];
                    }
                    else {
                        shortName = m[1];
                        longName = '%s, %s'.format(shortName, m[2]);
                    }
                }
                else {
                    shortName = name;
                    longName = name;
                }
                results.push({id:id, shortName:shortName, longName:longName});
            }
        }
        else {
            // Parse the search results
            var count = parseInt($('.content h1', frame.contentDocument).text());

            function pushResult(id, longName) {
                if ( id && longName ) {
                    var m = longName.match(/([\w\s,]+) \(Forecast for ([\w\s,]+)/i);
                    if ( m && m.length == 3 ) {
                        var shortName = m[1];
                        longName = "%s, %s".format(shortName, m[2]);
                    }
                    else
                        var shortName = longName.split(',')[0];
                    results.push({id:id, shortName:shortName, longName:longName});
                }
            };

            // Iterate over the first 10 results
            $('.linkList li a', frame.contentDocument).each(function() {
                var id = this.href.firstMatch(/\/mobile\/forecast\/([0-9]+)$/);
                pushResult(id, $(this).text());
            });

            // Then process further pages (if necessary)
            var resultsPerPage = parseInt(frame.contentDocument.URL.firstMatch(/itemsPerPage=([0-9]+)/i));
            if ( isNaN(resultsPerPage) )
                resultsPerPage = 10;

            if ( count > resultsPerPage ) {
                // Fetch further results via an async XMLHttpRequest, which we process via regex
                var startIndex = resultsPerPage;
                count -= resultsPerPage;

                var fetchNextSearchResults = null;
                
                function handleNextSearchResults(event, request) {
                    var ok = false;
                    if ( request.status == 200 && request.responseText ) {
                        try {
                            // Parse via regex (no xml).
                            // First get all the forecast links, then parse out the id and location
                            var m = request.responseText.match(/<a href="\/weather\/mobile\/forecast\/[0-9]+.*">.*<\/a>/ig);
                            for ( var i=0; i<m.length; ++i ) {
                                var mm = m[i].match(/\/([0-9]+).*>([\(\)\w\s,]+)</i);
                                if ( mm && mm.length == 3 )
                                    pushResult(mm[1], mm[2]);
                            }
                        }
                        catch(e) {
                            debug('handleNextSearchResults error: %s', e);
                        }
                    }
                    count -= resultsPerPage;
                    startIndex += resultsPerPage;

                    if ( count > 0 )
                        fetchNextSearchResults();
                    else
                        handleSearchResults(results);
                };

                fetchNextSearchResults = function() {
                    var nextPageUrl = frame.contentDocument.URL + '&startIndex=' + startIndex;
                    searchResultsRequest = createXMLRequest(nextPageUrl, handleNextSearchResults);
                    searchResultsRequest.send();
                };

                fetchNextSearchResults();
                return;
            }
        }
    }
    catch(e) {
        debug('Warning, search parse error: %s', e);
    }

    handleSearchResults(results);
}

function handleSearchResults(results) 
{
    searchResultsRequest = null;

    if ( results.length > 1 )
        showLocationsMenu(results);
    else if ( results.length == 1 )
        locationProcessed(results[0]);
    else
        locationProcessed({errorString:"No locations found"});
}

function showLocationsMenu(results)
{
    // Build the menu
    var menu = (window.widget ? window.widget.createMenu() : null);
    if ( menu ) {
        var i = 0;
        results.forEach( function(item) {
                menu.addMenuItem(item.longName);
                if ( item.id === undefined )
                    menu.setMenuItemEnabledAtIndex(i, false); // title
                ++i;
            } );

        // Show the menu
        var selection = menu.popup( menuPopupX, menuPopupY );
        if ( selection >= 0 && selection < results.length ) {
            locationProcessed(results[selection]);
        }
        else {
            locationProcessed({errorString:"No locations chosen"});
        }
    }
    else {
        // Just choose the first (non-title) result
        locationProcessed( (results.length > 1 ? results[1] : results[0]) );
    }
}

function locationProcessed( result )
{
    //debug('locationProcessed: %s, %s', result.id, result.shortName);

    isValidatingLocation = false;
    searchFrame().src = 'about:blank';

    if ( !result.errorString && result.id ) {
        // Get the location chooseice...
        prefs.locationId = result.id;
        prefs.locationShortName = (result.shortName ? result.shortName.strip() : "");
        prefs.locationLongName = (result.longName ? result.longName.strip() : "");
        storePrefs();

        endLocationValidation();
        updateWeatherData();
    } 
    else {
        debug( "Location processing error: " + result.errorString );
        endLocationValidation(result.errorString);
    }

    // If our done-button was clicked then hide prefs
    if ( prefsDoneClicked ) {
        $('#location-input').val(prefs.locationShortName); // Force locationValid so hidePrefs does not abort
        hidePrefs();
        prefsDoneClicked = false;
    }
}

function endLocationValidation( errorStr ) 
{
    if ( validateAnimTimerId ) {
        clearInterval( validateAnimTimerId );
        loadingAnimData[validateAnimTimerId] = undefined;
        validateAnimTimerId = 0;
    }

    // Reset the location caption
    $( "#location-caption" ).text("Enter a location or postcode:");

    // And set the location name
    if ( !errorStr )
        $('#location-input').val(prefs.locationShortName);
    else {
        // Let the user know something went wrong
        menu = (window.widget ? window.widget.createMenu() : null);
        if ( menu ) {
            menu.addMenuItem( errorStr );
            menu.setMenuItemEnabledAtIndex( 0, false );
            menu.popup( menuPopupX, menuPopupY );
        }
        else {
            $( "#location-input" ).val(errorStr);
        }
    }
}

function isLocationInputPending()
{
    return $('#location-input').val() != prefs.locationShortName;
}